type t = Stream of (char * t) option Lazy.t

let observe (Stream (lazy opt)) = opt

let get_current_char (Stream (lazy opt)) =
  Option.map ~f:fst opt

let move_forward (Stream (lazy opt)) =
  Option.map ~f:snd opt


let rec append (Stream first) (Stream second) =
  Stream (lazy (
      let reappend (char,middle) = (char, append middle (Stream second)) in
      Lazy.force first
      |> Option.map ~f:reappend
      |> Option.or_else (Lazy.force second)
    ))
let (++) = append

let rec of_string ?(offset=0) string =
  Stream (lazy (
      if offset >= String.length string then None
      else
        Some (String.get string offset,
              of_string ~offset:(offset+1) string)
    ))

let rec of_list list =
  Stream (lazy (
      match list with
      | [] -> None
      | head::others -> Some (head, of_list others)
    ))

let rec of_in_channel in_channel =
  Stream (lazy (
      try
        let c = input_char in_channel in
        Some (c, of_in_channel in_channel)
      with
        End_of_file -> None
    ))

let empty_closing in_channel =
  Stream (lazy (
      let () = close_in in_channel in
      None
    ))

let of_file ~filename =
  let in_channel = open_in filename in
  of_in_channel in_channel ++ empty_closing in_channel
