type 'elt bintree =
  | Leaf
  | Node of 'elt bintree * 'elt * 'elt bintree


open Parco

let make_node node (left,root,right) = Node (left,root,right)
let make_leaf leaf = Leaf

let read_tree elt =
  let elt = rule "element" elt in
  fix @@ fun tree ->
  rule "node" (make_node *> s "Node" <+> triple tree elt tree)
  || rule "leaf" (make_leaf *> s "Leaf") 


let res =
  "Node (Leaf, bob, Node (Node(Leaf,1,Leaf),5,Leaf)) unread"
  |> TextInput.of_string
  |> run (read_tree int)

let _ =
  Result.get_stack res 

    
type expr =
  | Litteral of int
  | Variable of string
  | Sum of expr * expr
  | Product of expr * expr

let string_of_list chars =
  chars
  |> List.map (StringLabels.make 1)
  |> StringLabels.concat ~sep:""


let make_litteral i = Litteral i
let make_variable str = Variable str
let make_product left _ right = Product (left,right)
let make_sum left _ right = Sum (left, right)

let variable =
  (fun letter word -> string_of_list (letter :: word))
  *> Char.lowercase
  <*> sequence Char.alphanumeric

let times = rule "product_operator" (s "*")
let plus = rule "sum operator" (s "+")
let lparen = rule "open parenthesis" (s "(")
let rparen = rule "close parenthesis" (s ")")

let rec expr () =
  fix @@ fun expr ->
  rule "sum" (make_sum *> term () <+> plus <+> expr)
  || rule "term" (term ())
and term () =
  fix @@ fun term ->
  rule "product" (make_product *> factor () <+> times <+> term)
  || rule "factor" (factor ())
and factor () =
  fix @@ fun factor ->
  rule "litteral" (make_litteral *> int)
  || rule "variable" (make_variable *> variable)
  || rule "parenthese" ((fun _ e _ -> e) *> lparen <+> expr () <+> rparen)
    

let res =
  "1 + 2 * (3 + foo baz * bar)"
  |> TextInput.of_string
  |> run (expr ())

let _ = Result.get_value res
let _ =
  match Result.get_stack res with
  | None -> Printf.printf "None.\n%!"
  | Some list ->
    List.iter (fun (string,pos) ->
        let open TextInput.Position in
        Printf.printf "%s: at %d:%d\n%!" string pos.line pos.column
      )
      list 
