module Position : sig
  type t =
    { description : string;
      column : int;
      line : int;
    }

  val compare : t -> t -> int
  val min : t -> t -> t
  val max : t -> t -> t

  val format : Format.formatter -> t -> unit
  val long_format : Format.formatter -> t -> unit
end

module TextInterval : sig
  type t = Position.t * Position.t

  val of_position : Position.t -> t
  val merge : t -> t -> t
  val (--) : t option -> t option -> t option

  val format : Format.formatter -> t -> unit
end


type t

val move_forward : t -> t option
val get_current_position : t -> Position.t
val get_current_char : t -> (char * Position.t) option
val observe : t -> ((char * Position.t) * t) option

val of_string : ?offset:int -> ?name:string -> string -> t
val of_list : ?name:string -> char list -> t
val of_file : filename:string -> t
val of_in_channel : ?name:string -> in_channel -> t
